use chrono::{DateTime, Utc};
use colored::*;
use std::fs;
use std::path::Path;
use std::process::exit;

use clap::Parser;
const VERSION: &str = clap::crate_version!();

#[derive(Parser)]
#[command(name = "hourglass", about = "Calculates the age of the system")]
struct Args {
    #[arg(short, long, value_name = "PATH")]
    path: Option<String>,

    #[arg(short, long, help = "Shows version")]
    version: bool,
}

const UNSUPPORTED_DIRECTORIES: [&str; 3] = ["/proc", "/dev", "/sys"];

fn count_system_age(path: String) -> Result<(), Box<dyn std::error::Error>> {
    if UNSUPPORTED_DIRECTORIES.contains(&path.as_str()) {
        println!("{} Directories such as /proc, /dev, /sys are not supported.", "ERROR:".bold().red());
        exit(-1);
    }

    let root_dir = if Path::new("/bedrock").exists() {
        "/home".to_string()
    } else if Path::new("/ostree").exists() {
        "/ostree".to_string()
    } else {
        path
    };

    let root_dir_path =  Path::new(&root_dir);

    if !root_dir_path.exists() {
        println!("{} No such file or directory!", "ERROR:".bold().red());
        exit(-1);
    }

    let metadata = fs::metadata(root_dir)?;


    let created = metadata.created()?;
    let created_datetime: DateTime<Utc> = created.into();

    let now = Utc::now();

    // Calculate the difference
    let duration = now.signed_duration_since(created_datetime);

    // Break down into years, months, and days
    let total_days = duration.num_days();
    let years = total_days / 365;
    let months = (total_days % 365) / 30;
    let days = (total_days % 365) % 30;

    let year = if years > 1 || years == 0 {
        "Years"
    } else {
        "Year"
    };

    let month = if months > 1 || months == 0 {
        "Months"
    } else {
        "Month"
    };

    let day = if days >= 2 || days == 0 {
        "Days"
    } else {
        "Day"
    };

    let months_string = months.to_string();
    let years_string = years.to_string();
    let days_string = days.to_string();

    println!(
        "{} {} {} {} {} {}",
        years_string.bold().yellow(),
        year.bold().yellow(),
        months_string.bold().green(),
        month.bold().green(),
        days_string.bold().cyan(),
        day.bold().cyan()
    );

    Ok(())
}

fn main() -> Result<(), Box<dyn std::error::Error>> {
    let args = Args::parse();

    if args.version == true {
        println!("{} {}", "Hourglass:".bold().blue(), VERSION);
        exit(0);
    };

    let _ = if let Some(path) = args.path {
        count_system_age(path)
    } else {
        count_system_age("/".to_string())
    };

    Ok(())
}
